%Author: Shebuti Rayana

%Zimek's method for converting outlier scores to probability estimates 

%input:
%Score - vector of scores which need to be unified and converted to probability estimates
% here it is considered that higher score means outlier and lower score
% means inlier

%distribution - distribution used to calculate the probability estimates
%e.g. - 'Gaussian' for Gaussian distribution, 'Exponential' for exponential
%distribution

%output:
%Pr - probability estimates for Score

function [Pr] = Unification(Score,distribution)

    mn = min(Score);
    mx = max(Score);
    
    if(abs(mn) > 0 && mx < Inf)
        temp = Regularize(Score);
        Score = temp;
    end

    mn = min(Score);
    mx = max(Score);

    if(isequal(mn,0) && mx > 1)
        temp = Normalize(Score);
        Score = temp;
    end

    %mn = min(Score);
    %mx = max(Score);

    %if(isequal(mn,0) && (mx <= 1))
        Pr = Unify(Score,distribution);
    %end
    
end

function [regScore] = Regularize(Score)
    [f,x] = hist(Score);
    mxF = max(f);
    ind = find(f == mxF);
    baseS = x(ind(1));
    regScore = Score - baseS;
    regScore(regScore<0) = 0;
end

function [normScore] = Normalize(Score)
    minS = min(Score);
    maxS = max(Score);
    normScore = (Score-minS)./(maxS-minS);
end

function [uniScore] = Unify(Score,tag)
    if(strcmp(tag,'Gaussian'))
        mu = mean(Score);
        sigma = sqrt(var(Score));
        temp = (Score-mu)./(sigma*sqrt(2));
        uniScore = erf(temp);
        uniScore(uniScore < 0) = 0;
    else if(strcmp(tag,'Exponential'))
            pd = fitdist(Score,tag);
            cdf = expcdf(Score,pd.mu);
            mucdf = expcdf(pd.mu,pd.mu);
            uniScore = (cdf-mucdf)./(1-mucdf);
            uniScore(uniScore < 0) = 0;
        else
            fprintf('Not a valid method');
        end
    end
end